// compile with gcc -Wall -g LCG_key_generation.c -lm -o LCG

#include <math.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>

static unsigned long int next = 1;

int rand(){
	next = (next * 1103515245 + 12345) & 0x7fffffff;
	return next;
}

void srand(unsigned int seed){
	next = seed;
}

void bits_byte(unsigned char val){
	int i; 
	for(i = 0; i < 8; i++)
		printf("%d", (val >> i) & 1);
}

void bits_int(unsigned int val){
	int i; 
	for(i = 0; i < 8*sizeof(int); i++)
		printf("%d", (val >> i) & 1);
	printf("\n");
}


void generate_key(unsigned char* array, int num_bytes){
	unsigned int rnd;
	int bit_offset, byte_offset, bits_used, i; 

	srand(time(NULL));
	memset(array, 0, num_bytes);	// clear all bytes  
	bits_used = 0;
	
	for(i = 0; i < num_bytes/sizeof(int); i++){
		rnd = rand();
		bits_int(rnd);
		
		bit_offset = bits_used % 8; 	// position of bit not set yet
		byte_offset = bits_used / 8;	// index of byte we will manipulate
		*((unsigned int*)(array + byte_offset)) |= (rnd << bit_offset);
		*((unsigned int*)(array + byte_offset + 1)) |= (rnd >>  ( 8 - bit_offset));	
		bits_used += 31; 
	}
	
	for(i = 0; i < num_bytes; i++){
		bits_byte(array[i]);
	}
}



int main (int argc, char *argv[])
{
  unsigned char *array;
  int num_bits = 256*8, num_bytes = ceil(num_bits / 8.0);
  array = (unsigned char *) malloc(num_bytes);
  generate_key(array, num_bytes);
  
  return 0;
}
