/*
	compile gcc RC4.c
	
	following code is remake of code from https://gist.github.com/rverton/a44fc8ca67ab9ec32089
    
	robin verton, dec 2015
    implementation of the RC4 algo

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define N 256   // 2^8

void swap(unsigned char *a, unsigned char *b) {
    int tmp = *a;
    *a = *b;
    *b = tmp;
}

void KSA(unsigned char *key, unsigned char *S, int keylength) {
    int j = 0;

    for(int i = 0; i < N; i++)
        S[i] = i;

    for(int i = 0; i < N; i++) {
        j = (j + S[i] + key[i % keylength]) % N;
        swap(&S[i], &S[j]);
    }
}

void PRGA(unsigned char *S, unsigned char *plaintext, unsigned char *ciphertext, int  text_size) {

    int i = 0, j = 0, k;
	unsigned char rnd;

    for(k = 0; k < text_size; k++) { 
        i = (i + 1) % N;
        j = (j + S[i]) % N;
        swap(&S[i], &S[j]);
        rnd = S[(S[i] + S[j]) % N];
        ciphertext[k] = rnd ^ plaintext[k];
        // printf("%2X ", rnd);
    }
}

void RC4(unsigned char *key, unsigned char *plaintext, unsigned char *ciphertext, int keylength, int text_size){
    unsigned char S[N];
    KSA(key, S, keylength);
    PRGA(S, plaintext, ciphertext, text_size);
}

void bytes_from_hex(unsigned char *array, unsigned char* hexstring, int num_bytes){
    unsigned char *pos = hexstring;
    int i; 
    for (i = 0; i < num_bytes; i++) {
        sscanf(pos, "%2hhx", &array[i]);
        pos += 2;
    }  
}

void bytes_as_hex(unsigned char *array, int size){
	int i; 
	for(i = 0; i < size; i++)
		printf("%02X", array[i]);
	printf("\n");
}

int main(int argc, char *argv[]) {

    if(argc != 3) {
        printf("Usage: %s <key size(num bits)> <plaintext (hex)>", argv[0]);
        return -1;
    }
	
  unsigned char *plaintext_bytes, *ciphertext_bytes, *key_bytes;
	int keylength, text_size;
	
	keylength = atoi(argv[1])/8;
	text_size = strlen(argv[2])/2;
	printf("keylength=%d, text_size=%d\n", keylength, text_size);
	
	plaintext_bytes = (unsigned char*)malloc(text_size);
	ciphertext_bytes = (unsigned char*)malloc(text_size);
	key_bytes = (unsigned char*)malloc(keylength);
	
	memset(key_bytes, 0, keylength);
	bytes_from_hex(plaintext_bytes, argv[2], text_size);
	
	bytes_as_hex(key_bytes, keylength);
	bytes_as_hex(plaintext_bytes, text_size);
	RC4(key_bytes, plaintext_bytes, ciphertext_bytes, keylength, text_size);
	bytes_as_hex(ciphertext_bytes, text_size);

    return 0;
}
